@@ -466,6 +466,13 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
     int hashes=0;
     unsigned int len2;
 
+    if (strlen(line) != len)
+    {
+	/* There must be an embedded null byte in the armor header */
+	log_error(_("invalid clearsig header\n"));
+	return -1;
+    }
+
     len2 = length_sans_trailing_ws ( line, len );
     if( !len2 ) {
         afx->buffer_pos = len2;  /* (it is not the fine way to do it here) */
@@ -513,7 +520,15 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
 	else if ((opt.compat_flags & COMPAT_ALLOW_NOT_DASH_ESCAPED)
                  && strlen (line) > 15
                  && !memcmp( line, "NotDashEscaped:", 15 ) )
-	  afx->not_dash_escaped = 1;
+	    {
+		for (unsigned int i=15; i<len; i++)
+		    if (!isprint(line[i]))
+		    {
+			log_error(_("invalid clearsig header\n"));
+			return -1;		 
+		    }
+	    	afx->not_dash_escaped = 1;
+	    }
 	else
 	  {
 	    log_error(_("invalid clearsig header\n"));
@@ -1054,7 +1069,13 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
         else if (afx->buffer_pos == 1 && c == '-'
                  && afx->buffer_len > 9
                  && !strncmp (afx->buffer, "-----END ", 9)) {
-            break; /* End in --dearmor mode or No CRC.  */
+            /* End in --dearmor mode.  */
+            if (n)
+                /* No CRC found, and the end of the armor is detected.
+                   Let return with the result of N-byte now, and come
+                   again to process the end of the armor with N=0.  */
+                afx->buffer_pos--;
+            break;
         }
 	else {
 	    log_error(_("invalid radix64 character %02X skipped\n"), c);
@@ -1578,11 +1599,16 @@ armor_filter( void *opaque, int control,
         /* Note that in a cleartext signature truncated lines in the
          * plaintext are detected and propagated to the signature
          * checking code by inserting a \f into the plaintext.  We do
-         * not use log_info here because some of the truncated lines
-         * are harmless.  */
-	if( afx->truncated )
-	    log_info(_("invalid armor: line longer than %d characters\n"),
-		      MAX_LINELEN );
+         * not use log_error here because some of the truncated lines
+         * are harmless, in compatibility mode.  */
+        if( afx->truncated ) {
+            if((opt.compat_flags & COMPAT_IGNORE_TRUNCATED_LINE))
+                log_info(_("invalid armor: line longer than %d characters\n"),
+                         MAX_LINELEN );
+            else
+                log_error(_("invalid armor: line longer than %d characters\n"),
+                          MAX_LINELEN );
+        }
 	/* issue an error to enforce dissemination of correct software */
 	if( afx->qp_detected )
 	    log_error(_("quoted printable character in armor - "
